;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xcebdfe2afebd16adfbdca3c2ba53b5ef . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xeee2bfe2d54f05c68a22ef49d02e7cef . "./code.cg2js/cg-printing.fasl")
(#x3a7b0205cbf5f73a05e8bd625b07ff7f . "./code.cg2js/cg-caret.fasl")
(#xa581a09006d869fb7599b49237b6af4d . "./code.cg2js/cg-item-list.fasl")
(#x4ff9c9d4ed1b2024e827bd26eab589d2 . "./code.cg2js/cg-progress-indicator.fasl")
(#x0b67c99eb55aaa2ba095619bca928883 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x12953f2ea1e64caefd7ce64656474057 . "./code.cg2js/cg-outline.fasl")
(#xa3be0f8b161eaf2d15e7d9bf228aff22 . "./code.cg2js/cg-palette.fasl")
(#x1b65be45bac5eaa33f92878cee715ce5 . "./code.cg2js/cg-selected-object.fasl")
(#xf891f337bf35c77bf356560d5416ddc5 . "./code.cg2js/cg-status-bar.fasl")
(#x5e74666bd8d58ff2d0a703251023c7bd . "./code.cg2js/cg-wrap-string.fasl")
(#xdd1cb3d7a16ea3c32dcf6ed7dfd9e196 . "./code.cg2js/cg-fill-texture.fasl")
(#xb477702c49b9a3a2ee6c379112867e6c . "./code.cg2js/cg-layout.fasl")
(#xfa5b3b873b581ef68a2caa31b85a6c8e . "./code.cg2js/cg-timer.fasl")
(#x2dc577ee6cc22440f69a14bc7afd9dbd . "./code.cg2js/cg-dialog-item.fasl")
(#x9a86e7b675fd5d41973c6a478a841e7e . "./code.cg2js/cg-button.fasl")
(#x88fed01c1ffbb02e2f0e1645210d21aa . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xbfb2790ccadce96e6cc8ac739717a889 . "./code.cg2js/cg-tray.fasl")
(#x186c9b5c47cfd009a59c12a5503f50e9 . "./code.cg2js/cg-plot-widget.fasl")
(#x2066503086d08346bb667747beffb0d2 . "./code.cg2js/cg-pixmap.fasl")
(#x2e1def26cc465ad38c03d56b4efc8aa3 . "./code.cg2js/cg-font-dialog.fasl")
(#x55153ad9a4c7705bffb98f4e6e9b34b5 . "./code.cg2js/cg-os-widget.fasl")
(#x897252971d45266e621f6dd21a1cc79b . "./code.cg2js/cg-text-or-combo.fasl")
(#x09c99bcd1975f9b3756845e54bd5ef9c . "./code.cg2js/cg-trackbar.fasl")
(#xa98224db2898b58574bdc5386283d50b . "./code.cg2js/cg-up-down-control.fasl")
(#x23c480b959bfb8dd5be1b0004f12dd47 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0b77c6db4b315820834d68fb89e657ae . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7e73363d3207331746d6c530e9d35944 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x06648ec27ec0a753f743a9123f478e16 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf5a885f7ed4af8e4c33310bd97efe1a9 . "./code.cg2js/cg-chart-widget.fasl")
(#x75ee77d21d444ec8bdf5508fbe0bbefa . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xe7edf43fb7e3958e35696c9bd9c4d464 . "./code.cg2js/cg-group-box.fasl")
(#xac17fa3215f5c96a880333e3b6424c1f . "./code.cg2js/cg-static-text.fasl")
(#xda36b3fc3ea44fbfb8d3968d35ac0211 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xaa97f191dace453910642e3e93bd2abb . "./code.cg2js/cg-color-dialog.fasl")
(#x75cda18d835c5603681be18bdfead076 . "./code.cg2js/cg-intersect.fasl")
(#x4b6a6f3873eae5321347803bc2492986 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x8c5f04c56a67f8452b1e2ca5098dc8db . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xeee4779ccba26ef31fdf7e15e84e365b . "./code.cg2js/cg-html-dialog.fasl")
(#x7a99183d998f75c2625626c1ac03f57f . "./code.cg2js/cg-lettered-menu.fasl")
(#x6cd796c69fbfd8efd6625d090adaa2b0 . "./code.cg2js/cg-class-grid.fasl")
(#x1d095155ddc2905646ff708a5648eb8a . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8cdf12105434ce827be445381dee434e . "./code.cg2js/cg-calendar.fasl")
(#x66b8c7d01448f15aec0515eb1e4e9341 . "./code.cg2js/cg-editable-text.fasl")
(#xf8d7872ab28f8c3b4fe5eb4bf0c9cbe4 . "./code.cg2js/cg-property.fasl")
(#x42f7ccbd3dde3a51e91412b90e768f13 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x350cf7f6539901d8451c7f8dfc4bb3e4 . "./code.cg2js/cg-grid-widget.fasl")
(#xb6c6057f5dfb904b0e7e4d644ef7822e . "./code.cg2js/cg-paren-matching.fasl")
(#x6949d16b791cad57a2d4656050997610 . "./code.cg2js/cg-dev.fasl")
(#xdfd38d217e133bb4d3119c2997c34fe8 . "./code.cg2js/cg-drawable.fasl")
(#x9b8a2285fa1b89e1137bc9407a7fa831 . "./code.cg2js/cg-curve.fasl")
(#x00417e1cdef8c05a9b65b725ee6c17c8 . "./code.cg2js/cg-lisp-widget.fasl")
(#xa52cfd908c88bab39ab25909c7c6d861 . "./code.cg2js/cg-dde-utils.fasl")
(#xfae2f2d9b9deb6fd714559df2f3401d3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xb9ca108652a936204c7672ec36cb7bdc . "./code.cg2js/cg-object-editor-layout.fasl")
(#x8cb2a80d565778b908d7b729c1cfc14e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x5557eaeabf9e1bac7088251fc05e36c6 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xfd9f739a20b83c06fda2628aa66f9672 . "./code.cg2js/cg-lamp.fasl")
(#xc6d85ac5aaed87ebca7fe978b01ccc45 . "./code.cg2js/cg-common-control.fasl")
(#x3d2c4d5a49d3a00f18be3aa0a789381d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2526cd97b0940946200d3482622f781b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb95145ad0b82b82fb92270a31eb7192f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x4edf7fe3bcc987ba9ac0b2475383f5e4 . "./code.cg2js/cg-project-window.fasl")
(#xb3b0aec75554d5ea02ef2832792cc3fa . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe0fecd81d828c1f33eb5117fc1e99bce . "./code.cg2js/cg-class-support.fasl")
(#x858478e926700c15059a2cca9f00c166 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe06e7210f4e7e5702222213640aa926e . "./code.cg2js/cg-yes-no-list.fasl")
(#x953c47b103820a963de5308fdc5df929 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6fb426f6a8d9f3db49dd5db57172d244 . "./code.cg2js/cg-string-dialog.fasl")
(#x394929623dc359e00ca9ddc73646917c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x71be1be93d707e7421d8d0eb45b71eb2 . "./code.cg2js/cg-dropping-outline.fasl")
(#x8057f098c39ee413db1186571f55ae30 . "./code.cg2js/cg.fasl")
(#x89713151b0b225c4b1d1ff735426fd58 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x45b0a00bb84b5e4276ec1e0de21e3290 . "./code.cg2js/cg-scaling-stream.fasl")
(#x4f7fdc3e02699d3596e9cc80b9319459 . "./code.cg2js/cg-yes-no-string.fasl")
(#x8957ff261e2d0a4ff5bfe7d210d40f8e . "./code.cg2js/cg-icon-pixmap.fasl")
(#x805389a3984baf50124703d403029b62 . "./code.cg2js/cg-web-browser.fasl")
(#xa7ff9dcac68db28c0a9117dbb5bc1ac1 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3057365f4c04bdec5cee354862d38729 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x5dcabf873c24cfc06a4794ce186c6e9b . "./code.cg2js/cg-message-dialog.fasl")
(#x5b74cf965c6320bf94ee3f9bfb5fc6b7 . "./code.cg2js/cg-comtab.fasl")
(#xc9d5269c233e6ece377bfe059f497291 . "./code.cg2js/cg-base.fasl")
(#x87ca20617b0a04a93a7b47f88cc89e78 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xefa1464c8b9f7eb87f864d97a482b1b1 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x36943dec829b40535d3761cde63c81ae . "./code.cg2js/cg-utility-dialog.fasl")
(#xbb9c2209427a8c9929a7ae5ec459aca7 . "./code.cg2js/cg-object-editor.fasl")
(#xbb21428ad0fccc15f6fa268f0977858e . "./code.cg2js/cg-edit-in-place.fasl")
(#xadaa2a8c1170b79187c3321c23b0041f . "./code.cg2js/cg-radio-button.fasl")
(#xcd907bbf821ea361955ace627cc71991 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xd447f14126ca57894e43f30001cae9b8 . "./code.cg2js/cg-nodes.fasl")
(#x4c5c76f5ba57f642dbd0b6709917b2ed . "./code.cg2js/cg-get-position.fasl")
(#x3d3721dd9c01c886a8e7a4a62c60e052 . "./code.cg2js/cg-directory-dialog.fasl")
(#xb35c1ff9168eb9633d5c9e47fe02feb0 . "./code.cg2js/cg-lisp-text.fasl")
(#xebbf242823ecd32824642ceb288d5ddf . "./code.cg2js/cg-combo-box.fasl")
(#x2d055cadb5a9a51815ee2097b42a08f7 . "./code.cg2js/cg-text-widget.fasl")
(#x102a531665141c28f1aa0e1b9e6b4fef . "./code.cg2js/cg-list-view.fasl")
(#x399cb7dd4e4ce6e9e18c63053f9d542a . "./code.cg2js/cg-graphics-context.fasl")
(#xba357de05890d60de6b5bcb56ae79b8c . "./code.cg2js/cg-get-pixmap.fasl")
(#x3937f642b324fcd11daf6c214770dd28 . "./code.cg2js/cg-menu.fasl")
(#x4e96a6a21311951b16de7944a65284c2 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x623d9a8c0a63ef8875801050f8cf056b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8b330feb5db231ee5053eff2061b1885 . "./code.cg2js/cg-tooltip.fasl")
(#xb924092567b2e94152e951775f1f3ad8 . "./code.cg2js/cg-html-widget.fasl")
(#x12f8e3491208496b066b2a5207fa1d2c . "./code.cg2js/cg-choice-list.fasl")
(#xd81ee6f7889dcea25849fd52d9a9eb5b . "./code.cg2js/cg-icon.fasl")
(#x4ec0add448b64379c2cc57a2d6e6ab20 . "./code.cg2js/cg-acache.fasl")
(#x11ba54b06fdd970ed550bb0be8625d31 . "./code.cg2js/cg-template-string.fasl")
(#xe27d82705c9d7ae69dcf71fac2d26ef4 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdb3769626850840d54b7968f9a4f06b4 . "./code.cg2js/cg-toggling-widget.fasl")
(#x9c0999d9e82bca17d518bbdd6932367e . "./code.cg2js/cg-tab-control.fasl")
(#xf87a84d2f2693766f02188dc940e2105 . "./code.cg2js/cg-split-bar.fasl")
(#xd53737a7f6049ea656bb60b5c2d31928 . "./code.cg2js/cg-picture-widget.fasl")
(#xdecfb795d4b843f045df6d73bfd46fe7 . "./code.cg2js/cg-player.fasl")
(#xb267bc08d34a95798d13d42172c17cd0 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x07e4f0c5bfa7130619793ed39e663bda . "./code.cg2js/cg-choose-printer.fasl")
(#x77f52d025fe55a6bf2f00fd9b04b45b8 . "./code.cg2js/cg-os-window.fasl")
(#x1361171b5b5c96c980b751a4e224f348 . "./code.cg2js/cg-scroll-bar.fasl")
(#xe4672b0e0124d0ea7388e5b62f3ead08 . "./code.cg2js/cg-clipboard.fasl")
(#xfc1a29a62cf4c2a79bad5b82635ef107 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x01cd960564c4e9bbfdb4dfe29ab7b4de . "./code.cg2js/cg-hotspot.fasl")
(#xb56dd06bbb032f7dc217f280f2918d61 . "./code.cg2js/cg-check-box.fasl")
(#x8e355f35ccc91710eb75deade2b19e3b . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x43d58d89416344515b696e9fe5bb027e . "./code.cg2js/cg-file-dialog.fasl")
(#x5578803a301d8a3d7bcb6f6136f15693 . "./code.cg2js/cg-toolbar.fasl")
(#x579243d5d269c418b38dcf2538a371ff . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xdf2418aa952cac1394e7bb150907905b . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa6d40b7096b18c1b7a5251796a680164 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x9e32ad917dc6fb6fde271d4aa1e8e034 . "./code.idejs/ide-project.fasl")
(#x65f6c4ca7c02820e346a3754b8dc9f0b . "./code.idejs/ide-intro.fasl")
(#x8a5e33e11fc37a911973446f4e660268 . "./code.idejs/ide-find-in-files.fasl")
(#x8cea087c0faa3eb63ee7b3dd87b97eaa . "./code.idejs/ide-grapher.fasl")
(#xb2ffb83e1924dd5cf518b4f19e312c4d . "./code.idejs/ide-inspector.fasl")
(#x9396ca994f54ddefab9d40cd2da85353 . "./code.idejs/ide-editor.fasl")
(#x5750ed75063b0ccb0b4cf28d9ebc8346 . "./code.idejs/ide-window-list.fasl")
(#xeefdf6a9c4d3c8c47b6f2ace93f90daf . "./code.idejs/ide-process-browser.fasl")
(#x042a59cc181eb83e3775ec29a042f578 . "./code.idejs/ide-patch-dialog.fasl")
(#x539b2634a2c8e35cb4c26d81f3663d20 . "./code.idejs/ide-class-grapher.fasl")
(#x19a2b5f695b311d0841dd3fe316a505c . "./code.idejs/ide-base.fasl")
(#x8071a53fbebdade307ccb1593e44db4a . "./code.idejs/ide.fasl")
(#xd2747aadda66262bca59412cd6096c4c . "./code.idejs/ide-trace.fasl")
(#x90d1fe4ecad90a9671ef0b307638bc0e . "./code.idejs/ide-form.fasl")
(#x3b9912cd2179f06694aa920a3eeff16d . "./code.idejs/ide-find-definitions.fasl")
(#x36e591acc7a89a5237f7d59f5a99131e . "./code.idejs/ide-apropos.fasl")
(#x3df08df331260221995e48a211f28d9c . "./code.idejs/ide-class-browser.fasl")
(#x76e8d25113f3b2754f9b5b9951816070 . "./code.idejs/ide-navigator.fasl")
(#x028b462378d805c4b58c8bba5bf999ac . "./code.idejs/ide-clipboard.fasl")
(#x1c1b4f3b82853e68fa90f6813be5c500 . "./code.idejs/ide-acache.fasl")
(#x0183efcc25ac412e117e20e18d5b9dce . "./code.idejs/ide-keyshort.fasl")
(#x78da1c61ccd5e4e1b7352f58861869b7 . "./code.idejs/ide-package-list.fasl")
(#x65485fc6c00a9aa49131dbc408c61e7c . "./code.idejs/ide-dev.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
